/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */

package org.olat.notifications;

import org.olat.core.id.OLATResourceable;
import org.olat.core.logging.OLog;
import org.olat.core.logging.Tracing;
import org.olat.core.util.notifications.Publisher;
import org.olat.core.util.resource.OresHelper;
import org.olat.course.CourseModule;
import org.olat.repository.RepositoryEntry;
import org.olat.repository.RepositoryManager;

/**
 * 
 * Description:<br>
 * 
 * <P>
 * Initial Date:  7 jan. 2010 <br>
 * @author srosse, stephane.rosse@frentix.com
 */
public class NotificationsUpgradeHelper {
	static OLog log = Tracing.createLoggerFor(NotificationsUpgradeHelper.class);

	public static boolean checkOLATResourceable(Publisher publisher) {
		Long resId = publisher.getResId();
		try {
			OLATResourceable ores = OresHelper.createOLATResourceableInstance(publisher.getResName(), resId);
			if(ores == null) return false;
			RepositoryEntry re = RepositoryManager.getInstance().lookupRepositoryEntry(ores, true);
			if(re == null) return false;
			return true;
		} catch (Exception e) {
			return false;
		}
	}
	
	public static boolean checkCourse(Publisher publisher) {
		Long resId = publisher.getResId();
		try {
			OLATResourceable ores = OresHelper.createOLATResourceableInstance(CourseModule.class, resId);
			if(ores == null) return false;
			RepositoryEntry re = RepositoryManager.getInstance().lookupRepositoryEntry(ores, true);
			if(re == null) return false;
			return true;
		} catch (Exception e) {
			return false;
		}
	}
	
	public static String getCourseNodePath(Publisher publisher) {
		String businessPath = null;
		try {
			RepositoryEntry re = RepositoryManager.getInstance().lookupRepositoryEntry(OresHelper.createOLATResourceableInstance(CourseModule.class, publisher.getResId()), true);
			businessPath = "[RepositoryEntry:" + re.getKey() + "][CourseNode:" + publisher.getSubidentifier() + "]";
		} catch (Exception e) {
			//if something went wrong...
			log.warn("error while processing resid: "+publisher.getResId(), e);
		}
		return businessPath;
	}
}
